@@ -1,8 +1,10 @@
+from types import FunctionType
+
+import jax
+import jax.lax as lax
 import jax.numpy as jnp
 import jax.scipy.special as jsps
-import jax.lax as lax
-import jax
-from types import FunctionType
+from plum import Union
 
 from . import B, dispatch, Numeric
 from .custom import jax_register
@@ -12,18 +14,18 @@ from ..types import JAXDType, JAXNumeric, NPNumeric, Number, Int
 __all__ = []
 
 
-@dispatch(Numeric)
-def isnan(a):
+@dispatch
+def isnan(a: Numeric):
     return jnp.isnan(a)
 
 
-@dispatch(JAXNumeric)
-def device(a):
+@dispatch
+def device(a: JAXNumeric):
     return str(a.device_buffer.device())
 
 
-@dispatch(JAXNumeric)
-def move_to_active_device(a):
+@dispatch
+def move_to_active_device(a: JAXNumeric):
     if B.Device.active_name:
         parts = B.Device.active_name.lower().split(":")
         if len(parts) == 1:
@@ -38,211 +40,217 @@ def move_to_active_device(a):
         return a
 
 
-@dispatch(JAXDType, [Int])
-def zeros(dtype, *shape):
+@dispatch
+def zeros(dtype: JAXDType, *shape: Int):
     return move_to_active_device(jnp.zeros(shape, dtype=dtype))
 
 
-@dispatch(JAXDType, [Int])
-def ones(dtype, *shape):
+@dispatch
+def ones(dtype: JAXDType, *shape: Int):
     return move_to_active_device(jnp.ones(shape, dtype=dtype))
 
 
-@dispatch(JAXDType, Int, Int)
-def eye(dtype, *shape):
+@dispatch
+def _eye2(dtype: JAXDType, *shape: Int):
     return move_to_active_device(jnp.eye(shape[0], shape[1], dtype=dtype))
 
 
-@dispatch(JAXDType, object, object, Int)
-def linspace(dtype, a, b, num):
+@dispatch
+def linspace(dtype: JAXDType, a, b, num: Int):
     return move_to_active_device(jnp.linspace(a, b, num, dtype=dtype))
 
 
-@dispatch(JAXDType, object, object, object)
-def range(dtype, start, stop, step):
+@dispatch
+def range(dtype: JAXDType, start, stop, step):
     return move_to_active_device(jnp.arange(start, stop, step, dtype=dtype))
 
 
-@dispatch(JAXDType, JAXNumeric)
-def cast(dtype, a):
+@dispatch
+def cast(dtype: JAXDType, a: JAXNumeric):
     return a.astype(dtype)
 
 
-@dispatch(JAXDType, {Number, NPNumeric})
-def cast(dtype, a):
+@dispatch
+def cast(dtype: JAXDType, a: Union[Number, NPNumeric]):
     return move_to_active_device(jnp.array(a, dtype=dtype))
 
 
-@dispatch(Numeric)
-def identity(a):
+@dispatch
+def identity(a: Numeric):
     # Do not return `a` identically.
     return 1 * a
 
 
-@dispatch(Numeric)
-def negative(a):
+@dispatch
+def negative(a: Numeric):
     return jnp.negative(a)
 
 
-@dispatch(Numeric)
-def abs(a):
+@dispatch
+def abs(a: Numeric):
     return jnp.abs(a)
 
 
-@dispatch(Numeric)
-def sign(a):
+@dispatch
+def sign(a: Numeric):
     return jnp.sign(a)
 
 
-@dispatch(Numeric)
-def sqrt(a):
+@dispatch
+def sqrt(a: Numeric):
     return jnp.sqrt(a)
 
 
-@dispatch(Numeric)
-def exp(a):
+@dispatch
+def exp(a: Numeric):
     return jnp.exp(a)
 
 
-@dispatch(Numeric)
-def log(a):
+@dispatch
+def log(a: Numeric):
     return jnp.log(a)
 
 
-@dispatch(Numeric)
-def sin(a):
+@dispatch
+def sin(a: Numeric):
     return jnp.sin(a)
 
 
-@dispatch(Numeric)
-def cos(a):
+@dispatch
+def cos(a: Numeric):
     return jnp.cos(a)
 
 
-@dispatch(Numeric)
-def tan(a):
+@dispatch
+def tan(a: Numeric):
     return jnp.tan(a)
 
 
-@dispatch(Numeric)
-def tanh(a):
+@dispatch
+def tanh(a: Numeric):
     return jnp.tanh(a)
 
 
-@dispatch(Numeric)
-def erf(a):
+@dispatch
+def erf(a: Numeric):
     return jsps.erf(a)
 
 
-@dispatch(Numeric, Numeric)
-def add(a, b):
+@dispatch
+def add(a: Numeric, b: Numeric):
     return jnp.add(a, b)
 
 
-@dispatch(Numeric, Numeric)
-def subtract(a, b):
+@dispatch
+def subtract(a: Numeric, b: Numeric):
     return jnp.subtract(a, b)
 
 
-@dispatch(Numeric, Numeric)
-def multiply(a, b):
+@dispatch
+def multiply(a: Numeric, b: Numeric):
     return jnp.multiply(a, b)
 
 
-@dispatch(Numeric, Numeric)
-def divide(a, b):
+@dispatch
+def divide(a: Numeric, b: Numeric):
     return jnp.divide(a, b)
 
 
-@dispatch(Numeric, Numeric)
-def power(a, b):
+@dispatch
+def power(a: Numeric, b: Numeric):
     return jnp.power(a, b)
 
 
-@dispatch(Numeric, Numeric)
-def minimum(a, b):
+@dispatch
+def minimum(a: Numeric, b: Numeric):
     return jnp.minimum(a, b)
 
 
-@dispatch(Numeric, Numeric)
-def maximum(a, b):
+@dispatch
+def maximum(a: Numeric, b: Numeric):
     return jnp.maximum(a, b)
 
 
-@dispatch(Numeric)
-def min(a, axis=None):
+@dispatch
+def min(a: Numeric, axis=None):
     return jnp.min(a, axis=axis)
 
 
-@dispatch(Numeric)
-def max(a, axis=None):
+@dispatch
+def max(a: Numeric, axis=None):
     return jnp.max(a, axis=axis)
 
 
-@dispatch(Numeric)
-def sum(a, axis=None):
+@dispatch
+def sum(a: Numeric, axis=None):
     return jnp.sum(a, axis=axis)
 
 
-@dispatch(Numeric)
-def mean(a, axis=None):
+@dispatch
+def mean(a: Numeric, axis=None):
     return jnp.mean(a, axis=axis)
 
 
-@dispatch(Numeric)
-def std(a, axis=None):
+@dispatch
+def std(a: Numeric, axis=None):
     return jnp.std(a, axis=axis, ddof=0)
 
 
-@dispatch(Numeric)
-def all(a, axis=None):
+@dispatch
+def all(a: Numeric, axis=None):
     return jnp.all(a, axis=axis)
 
 
-@dispatch(Numeric)
-def any(a, axis=None):
+@dispatch
+def any(a: Numeric, axis=None):
     return jnp.any(a, axis=axis)
 
 
-@dispatch(Numeric, Numeric)
-def lt(a, b):
+@dispatch
+def lt(a: Numeric, b: Numeric):
     return jnp.less(a, b)
 
 
-@dispatch(Numeric, Numeric)
-def le(a, b):
+@dispatch
+def le(a: Numeric, b: Numeric):
     return jnp.less_equal(a, b)
 
 
-@dispatch(Numeric, Numeric)
-def gt(a, b):
+@dispatch
+def gt(a: Numeric, b: Numeric):
     return jnp.greater(a, b)
 
 
-@dispatch(Numeric, Numeric)
-def ge(a, b):
+@dispatch
+def ge(a: Numeric, b: Numeric):
     return jnp.greater_equal(a, b)
 
 
-f = jax_register(bvn_cdf, s_bvn_cdf)
-dispatch(Numeric, Numeric, Numeric)(f)
+_bvn_cdf = jax_register(bvn_cdf, s_bvn_cdf)
 
 
-@dispatch(Numeric, FunctionType, FunctionType, JAXNumeric, [JAXNumeric])
-def _cond(condition, f_true, f_false, *xs):
+@dispatch
+def bvn_cdf(a: Numeric, b: Numeric, c: Numeric):
+    return _bvn_cdf(a, b, c)
+
+
+@dispatch
+def _cond(
+    condition: Numeric, f_true: FunctionType, f_false: FunctionType, *xs: JAXNumeric
+):
     return lax.cond(condition, lambda xs_: f_true(*xs_), lambda xs_: f_false(*xs_), xs)
 
 
-@dispatch(Numeric)
-def sort(a, axis=-1, descending=False):
+@dispatch
+def sort(a: Numeric, axis=-1, descending=False):
     if descending:
         return -jnp.sort(-a, axis=axis)
     else:
         return jnp.sort(a, axis=axis)
 
 
-@dispatch(Numeric)
-def argsort(a, axis=-1, descending=False):
+@dispatch
+def argsort(a: Numeric, axis=-1, descending=False):
     if descending:
         return jnp.argsort(-a, axis=axis)
     else:
