@@ -1,10 +1,12 @@
-import numpy as np
 import math
 
-from . import dispatch
+import numpy as np
+from plum import Union
+
+from . import B, dispatch
+from .shape import Shape
 from .types import Numeric, Int
 from .util import abstract
-from .shape import Shape
 
 __all__ = [
     "lazy_shapes",
@@ -17,6 +19,8 @@ __all__ = [
     "squeeze",
     "uprank",
     "diag",
+    "diag_extract",
+    "diag_construct",
     "flatten",
     "vec_to_tril",
     "tril_to_vec",
@@ -50,8 +54,8 @@ class LazyShapes:
 lazy_shapes = LazyShapes()  #: Enable lazy shapes.
 
 
-@dispatch(Numeric)
-def shape(a):  # pragma: no cover
+@dispatch
+def shape(a: Numeric):  # pragma: no cover
     """Get the shape of a tensor.
 
     Args:
@@ -70,13 +74,13 @@ def shape(a):  # pragma: no cover
         return ()
 
 
-@dispatch({list, tuple})
-def shape(a):
+@dispatch
+def shape(a: Union[list, tuple]):
     return np.array(a).shape
 
 
-@dispatch({Numeric, list, tuple})
-def rank(a):  # pragma: no cover
+@dispatch
+def rank(a: Union[Numeric, list, tuple]):  # pragma: no cover
     """Get the shape of a tensor.
 
     Args:
@@ -88,9 +92,9 @@ def rank(a):  # pragma: no cover
     return len(shape(a))
 
 
-@dispatch(Numeric)
+@dispatch
 @abstract()
-def length(a):  # pragma: no cover
+def length(a: Numeric):  # pragma: no cover
     """Get the length of a tensor.
 
     Args:
@@ -104,8 +108,8 @@ def length(a):  # pragma: no cover
 size = length  #: Alias for `length`.
 
 
-@dispatch(Numeric)
-def isscalar(a):
+@dispatch
+def isscalar(a: Numeric):
     """Check whether a tensor is a scalar.
 
     Args:
@@ -117,9 +121,9 @@ def isscalar(a):
     return rank(a) == 0
 
 
-@dispatch(Numeric)
+@dispatch
 @abstract()
-def expand_dims(a, axis=0):  # pragma: no cover
+def expand_dims(a: Numeric, axis=0):  # pragma: no cover
     """Insert an empty axis.
 
     Args:
@@ -131,9 +135,9 @@ def expand_dims(a, axis=0):  # pragma: no cover
     """
 
 
-@dispatch(Numeric)
+@dispatch
 @abstract()
-def squeeze(a):  # pragma: no cover
+def squeeze(a: Numeric):  # pragma: no cover
     """Remove all axes containing only a single element.
 
     Args:
@@ -144,36 +148,35 @@ def squeeze(a):  # pragma: no cover
     """
 
 
-@dispatch({tuple, list})
-def squeeze(a):
+@dispatch
+def squeeze(a: Union[tuple, list]):
     if len(a) == 1:
         return a[0]
     else:
         return a
 
 
-@dispatch(Numeric)
-def uprank(a):
-    """Convert the input into a rank two tensor.
+@dispatch
+def uprank(a: Numeric, rank=2):
+    """Convert the input into a tensor of at least rank `rank`.
 
     Args:
         a (tensor): Tensor.
+        rank (int, optional): Rank. Defaults to `2`.
 
     Returns:
         tensor: `a`, but of rank two.
     """
-    a_rank = rank(a)
-    if a_rank > 2:
-        raise ValueError(f"Cannot convert a tensor of rank {a_rank} to rank 2.")
-    while a_rank < 2:
+    a_rank = B.rank(a)
+    while a_rank < rank:
         a = expand_dims(a, axis=-1)
         a_rank += 1
     return a
 
 
-@dispatch(Numeric)
+@dispatch
 @abstract()
-def diag(a):  # pragma: no cover
+def diag(a: Numeric):
     """Take the diagonal of a matrix, or construct a diagonal matrix from its
     diagonal.
 
@@ -185,8 +188,46 @@ def diag(a):  # pragma: no cover
     """
 
 
-@dispatch(Numeric)
-def flatten(a):  # pragma: no cover
+@dispatch
+@abstract()
+def diag_extract(a: Numeric):  # pragma: no cover
+    """Take the diagonal of a matrix.
+
+    Args:
+        a (tensor): Matrix.
+
+    Returns:
+        tensor: Diagonal of matrix.
+    """
+
+
+@dispatch
+def diag_construct(a: Numeric):  # pragma: no cover
+    """Construct a diagonal matrix from its diagonal.
+
+    Args:
+        a (tensor): Diagonal.
+
+    Returns:
+        tensor: Matrix.
+    """
+    if B.rank(a) == 0:
+        raise ValueError("Input must have at least rank 1.")
+
+    # The one-dimensional case is better handled by `diag`.
+    if B.rank(a) == 1:
+        return B.diag(a)
+
+    identity_matrix = B.eye(B.dtype(a), B.shape(a)[-1])
+    # Deal with the batch dimensions.
+    for i in range(B.rank(a) - 1):
+        identity_matrix = B.expand_dims(identity_matrix, axis=0)
+    # Use broadcasting to get the desired output.
+    return B.expand_dims(a, axis=-1) * identity_matrix
+
+
+@dispatch
+def flatten(a: Numeric):  # pragma: no cover
     """Flatten a tensor.
 
     Args:
@@ -198,9 +239,8 @@ def flatten(a):  # pragma: no cover
     return reshape(a, -1)
 
 
-def _vec_to_tril_shape_upper_perm(a, offset=0):
+def _vec_to_tril_side_upper_perm(m, offset=0):
     # Compute the length of a side of the square result.
-    m = shape(a)[0]
     k = offset
     if k <= 0:
         side = int((math.sqrt(1 + 8 * m) - 1) / 2) - k
@@ -219,9 +259,8 @@ def _vec_to_tril_shape_upper_perm(a, offset=0):
     return side, len(ind_upper[0]), perm
 
 
-@dispatch(Numeric)
-@abstract()
-def vec_to_tril(a, offset=0):  # pragma: no cover
+@dispatch
+def vec_to_tril(a: Numeric, offset=0):
     """Construct a lower triangular matrix from a vector.
 
     Args:
@@ -231,11 +270,16 @@ def vec_to_tril(a, offset=0):  # pragma: no cover
     Returns:
         tensor: Lower triangular matrix.
     """
+    if B.rank(a) < 1:
+        raise ValueError("Input must be at least rank 1.")
+    batch_shape = B.shape(a)[:-1]
+    side, upper, perm = _vec_to_tril_side_upper_perm(B.shape(a)[-1], offset=offset)
+    a = B.concat(a, B.zeros(B.dtype(a), *batch_shape, upper), axis=-1)
+    return B.reshape(B.take(a, perm, axis=-1), *batch_shape, side, side)
 
 
-@dispatch(Numeric)
-@abstract()
-def tril_to_vec(a, offset=0):  # pragma: no cover
+@dispatch
+def tril_to_vec(a: Numeric, offset=0):
     """Construct a vector from a lower triangular matrix.
 
     Args:
@@ -245,9 +289,19 @@ def tril_to_vec(a, offset=0):  # pragma: no cover
     Returns:
         tensor: Vector
     """
+    if B.rank(a) < 2:
+        raise ValueError("Input must be at least rank 2.")
+    batch_shape = B.shape(a)[:-2]
+    n, m = B.shape(a)[-2:]
+    if n != m:
+        raise ValueError("Input must be square.")
+    indices = np.tril_indices(n, k=offset)
+    # Convert to linear indices to be able to use `B.take`.
+    indices = n * indices[0] + indices[1]
+    return B.take(B.reshape(a, *batch_shape, n * n), indices, axis=-1)
 
 
-@dispatch([object])
+@dispatch
 @abstract()
 def stack(*elements, **kw_args):  # pragma: no cover
     """Concatenate tensors along a new axis.
@@ -261,9 +315,9 @@ def stack(*elements, **kw_args):  # pragma: no cover
     """
 
 
-@dispatch(Numeric)
+@dispatch
 @abstract()
-def unstack(a, axis=0):  # pragma: no cover
+def unstack(a: Numeric, axis=0):  # pragma: no cover
     """Unstack tensors along an axis.
 
     Args:
@@ -275,9 +329,9 @@ def unstack(a, axis=0):  # pragma: no cover
     """
 
 
-@dispatch(Numeric, [Int])
+@dispatch
 @abstract()
-def reshape(a, *shape):  # pragma: no cover
+def reshape(a: Numeric, *shape: Int):  # pragma: no cover
     """Reshape a tensor.
 
     Args:
@@ -289,7 +343,7 @@ def reshape(a, *shape):  # pragma: no cover
     """
 
 
-@dispatch([object])
+@dispatch
 @abstract()
 def concat(*elements, **kw_args):  # pragma: no cover
     """Concatenate tensors along an axis.
@@ -303,8 +357,8 @@ def concat(*elements, **kw_args):  # pragma: no cover
     """
 
 
-@dispatch([{list, tuple}])
-def concat2d(*rows):
+@dispatch
+def concat2d(*rows: Union[list]):
     """Concatenate tensors into a matrix.
 
     Args:
@@ -317,9 +371,9 @@ def concat2d(*rows):
     return concat(*[concat(*row, axis=1) for row in rows], axis=0)
 
 
-@dispatch(Numeric, [Int])
+@dispatch
 @abstract()
-def tile(a, *repeats):  # pragma: no cover
+def tile(a: Numeric, *repeats: Int):  # pragma: no cover
     """Tile a tensor.
 
     Args:
@@ -331,9 +385,8 @@ def tile(a, *repeats):  # pragma: no cover
     """
 
 
-@dispatch(Numeric, object)
-@abstract(promote=None)
-def take(a, indices_or_mask, axis=0):  # pragma: no cover
+@dispatch
+def take(a: Numeric, indices_or_mask, axis=0):  # pragma: no cover
     """Take particular elements along an axis.
 
     Args:
@@ -345,3 +398,14 @@ def take(a, indices_or_mask, axis=0):  # pragma: no cover
     Returns:
         tensor: Selected subtensor.
     """
+    if B.rank(indices_or_mask) != 1:
+        raise ValueError("Indices or mask must be rank 1.")
+    # Resolve negative axis specification.
+    if axis < 0:
+        axis += B.rank(a)
+    # Construct slices.
+    slices = tuple(
+        indices_or_mask if i == axis else slice(None, None, None)
+        for i in range(B.rank(a))
+    )
+    return a[slices]
